package types

import (
	"database/sql"
	"database/sql/driver"
	"encoding/json"
	"fmt"
	"time"

	"go.uber.org/zap"
)

var (
	loc             *time.Location = time.FixedZone("+0800", 8*60*60)
	datetimeLayouts                = []string{
		"2006-01-02 15:04:05", "2006-1-2 15:04:05", "2006/01/02 15:04:05", "06-1-2 15:04:05", "06-01-02 15:04:05", "01/02/06 15:04:05", "1/2/06 15:04:05", "2006年01月02日 15:04:05", "06年1月2日 15:04:05",
		"2006-01-02 15:04", "2006-1-2 15:04", "2006/01/02 15:04", "06-1-2 15:04", "06-01-02 15:04", "01/02/06 15:04", "1/2/06 15:04", "2006年01月02日 15:04", "06年1月2日 15:04",
	}
)

type DateTime struct {
	time.Time
}

func Now() DateTime {
	return DateTime{
		Time: time.Now().In(loc),
	}
}

func NewEmptyDateTime() DateTime {
	return DateTime{
		Time: time.Time{}.In(loc),
	}
}

func MinDateTime() DateTime {
	return DateTime{
		Time: time.Date(1, 1, 1, 0, 0, 0, 0, loc),
	}
}

func MaxDateTime() DateTime {
	return DateTime{
		Time: time.Date(9999, 12, 31, 23, 59, 59, 999999, loc),
	}
}

func Timestamp() int64 {
	startline := time.Date(2022, 2, 22, 22, 22, 22, 0, loc).Unix()
	return Now().Unix() - startline
}

func FromTime(t time.Time) DateTime {
	return DateTime{
		Time: t,
	}
}

func FromUnixMicro(sec int64) DateTime {
	return DateTime{
		Time: time.UnixMicro(sec).In(loc),
	}
}

func ParseDateTime(t string) (DateTime, error) {
	if len(t) == 0 {
		return NewEmptyDateTime(), fmt.Errorf("不能解析空白的日期时间。")
	}
	for _, layout := range datetimeLayouts {
		fmt.Printf("Parse: %s, Try layout: %s\n", t, layout)
		d, err := time.ParseInLocation(layout, t, loc)
		if err == nil {
			return DateTime{
				Time: d,
			}, nil
		}
	}
	return NewEmptyDateTime(), fmt.Errorf("无法解析给定的日期时间，格式不正确。")
}

func ParseDateTimep(t string) (*DateTime, error) {
	if len(t) == 0 {
		return nil, fmt.Errorf("不能解析空白的日期时间。")
	}
	for _, layout := range datetimeLayouts {
		fmt.Printf("Parse: %s, Try layout: %s\n", t, layout)
		d, err := time.ParseInLocation(layout, t, loc)
		if err == nil {
			return &DateTime{
				Time: d,
			}, nil
		}
	}
	return nil, fmt.Errorf("无法解析给定的日期时间，格式不正确。")
}

var _ driver.Valuer = (*DateTime)(nil)

func (dt DateTime) Value() (driver.Value, error) {
	return dt.In(loc).Format("2006-01-02 15:04:05"), nil
}

var _ sql.Scanner = (*DateTime)(nil)

func (dt *DateTime) Scan(src interface{}) (err error) {
	switch src := src.(type) {
	case time.Time:
		dt.Time = src
	case string:
		t, err := time.ParseInLocation("2006-01-02 15:04:05", src, loc)
		if err != nil {
			return err
		}
		*dt = DateTime{Time: t}
	case []byte:
		dt.Time, err = time.ParseInLocation("2006-01-02 15:04:05", string(src), loc)
		return err
	case nil:
		dt = nil
	default:
		return fmt.Errorf("该数据类型不支持解析到日期时间: %T", src)
	}
	return nil
}

var _ json.Marshaler = (*DateTime)(nil)

func (dt DateTime) MarshalJSON() ([]byte, error) {
	return json.Marshal(dt.Format("2006-01-02 15:04:05"))
}

var _ json.Unmarshaler = (*DateTime)(nil)

func (dt *DateTime) UnmarshalJSON(data []byte) error {
	var str string
	if err := json.Unmarshal(data, &str); err != nil {
		return fmt.Errorf("不能解析指定的日期时间值: %w", err)
	}
	t, err := time.ParseInLocation("2006-01-02 15:04:05", str, loc)
	dt.Time = t
	return err
}

func (dt DateTime) DifferenceInMonth(d DateTime) int {
	var differYear, differMonth int
	differYear = dt.Year() - d.Year()
	differMonth = int(dt.Month() - d.Month())
	return differYear*12 + differMonth
}

func (dt DateTime) IsNextMonth(target DateTime) bool {
	return dt.DifferenceInMonth(target) == 1
}

func (dt *DateTime) ToBeginningOfDate() {
	dt.Time = time.Date(dt.Year(), dt.Month(), dt.Day(), 0, 0, 0, 0, loc)
}

func (dt *DateTime) ToEndingOfDate() {
	dt.Time = time.Date(dt.Year(), dt.Month(), dt.Day(), 23, 59, 59, 999999, loc)
}

func (dt DateTime) Date() Date {
	return Date{
		Time: time.Date(dt.Year(), dt.Month(), dt.Day(), 0, 0, 0, 0, loc),
	}
}

func (dt DateTime) IsEmpty() bool {
	return dt.Time.IsZero()
}

func (dt *DateTime) Parse(s string) error {
	t, err := ParseDateTime(s)
	if err != nil {
		return err
	}
	dt.Time = t.Time
	return nil
}

func (dt DateTime) ToString() string {
	return dt.Time.Format("2006-01-02 15:04:05")
}

func (dt DateTime) Log(fieldName string) zap.Field {
	return zap.String(fieldName, dt.ToString())
}
