package types

import (
	"errors"
	"strings"

	"github.com/jackc/pgx/v5/pgtype"
)

type Parse interface {
	Parse(string) error
}

type ToString interface {
	ToString() string
}

// 将一个字符串拆解解析为一个 Postgresql 范围类型的值。
func destructureToRange[T any, PT interface {
	Parse
	*T
}](s string) (pgtype.Range[T], error) {
	var r pgtype.Range[T]
	r.Valid = false
	if len(s) == 0 {
		r.LowerType = pgtype.Empty
		r.UpperType = pgtype.Empty
		return r, nil
	}
	rangeUnit := strings.Split(s, ",")
	if len(rangeUnit) != 2 {
		return r, errors.New("无法解析给定的范围值，格式不正确。")
	}
	if unit, found := strings.CutPrefix(rangeUnit[0], "["); found {
		var t PT
		if len(unit) > 0 {
			r.LowerType = pgtype.Inclusive
			err := t.Parse(unit)
			if err != nil {
				return r, errors.New("无法解析给定的最低范围值。")
			}
		} else {
			r.LowerType = pgtype.Unbounded
		}
		r.Lower = *t
	}
	if unit, found := strings.CutPrefix(rangeUnit[0], "("); found {
		var t PT
		if len(unit) > 0 {
			r.LowerType = pgtype.Exclusive
			err := t.Parse(unit)
			if err != nil {
				return r, errors.New("无法解析给定的最低范围值。")
			}
		} else {
			r.LowerType = pgtype.Unbounded
		}
		r.Lower = *t
	}

	if unit, found := strings.CutSuffix(rangeUnit[1], "]"); found {
		var t PT
		if len(unit) > 0 {
			r.UpperType = pgtype.Inclusive
			err := t.Parse(unit)
			if err != nil {
				return r, errors.New("无法解析给定的最高范围值。")
			}
		} else {
			r.UpperType = pgtype.Unbounded
		}
		r.Upper = *t
	}
	if unit, found := strings.CutSuffix(rangeUnit[1], ")"); found {
		var t PT
		if len(unit) > 0 {
			r.UpperType = pgtype.Exclusive
			err := t.Parse(unit)
			if err != nil {
				return r, errors.New("无法解析给定的最高范围值。")
			}
		} else {
			r.UpperType = pgtype.Unbounded
		}
		r.Upper = *t
	}
	r.Valid = true
	return r, nil
}

// 将一个范围类型的值转换为一个字符串、
func assembleRange[T ToString](r pgtype.Range[T]) string {
	var sb strings.Builder
	if r.LowerType == pgtype.Empty || r.UpperType == pgtype.Empty {
		return "empty"
	}
	if r.LowerType == pgtype.Inclusive {
		sb.WriteString("[")
	} else {
		sb.WriteString("(")
	}
	if r.LowerType != pgtype.Unbounded {
		sb.WriteString(r.Lower.ToString())
	}
	sb.WriteString(",")
	if r.UpperType != pgtype.Unbounded {
		sb.WriteString(r.Upper.ToString())
	}
	if r.UpperType == pgtype.Inclusive {
		sb.WriteString("]")
	} else {
		sb.WriteString(")")
	}
	return sb.String()
}
